<script setup lang="ts">
import ClientSelect from "@/components/ClientSelect.vue";
import SearchTable from "@/components/SearchTable.vue";
import UserSelect from "@/components/UserSelect.vue";
import useSearchForm from "@/composables/useSearchForm";
import userActivityLogPresenter from "@/presenter/user-activity-log";
import userActivityLogQuery from "@/queries/user-activity-log";
import { IListUserActivityLogQuery } from "@/types/api/user-activity-log";
import { Icon } from "@iconify/vue";
import { keepPreviousData, useQuery } from "@tanstack/vue-query";

import {
  ElButton,
  ElDatePicker,
  ElForm,
  ElFormItem,
  ElInput,
  ElOption,
  ElSelect,
  ElTableColumn,
  ElTag,
  FormInstance,
} from "element-plus";
import { ref } from "vue";
import { useI18n } from "vue-i18n";

const { d, t } = useI18n();

const { page, perPage, perPages, form, submitForm, search } =
  useSearchForm<IListUserActivityLogQuery>({
    ip: undefined,
    userIds: [],
    clientIds: [],
    eventTypes: [],
    occurredStartAt: undefined,
    occurredEndAt: undefined,
  });

const typeOptions = userActivityLogPresenter.eventTypeElOptions();
const formEl = ref<FormInstance>();

const resetForm = () => {
  formEl.value?.resetFields();
  search();
};

const { isFetching, data } = useQuery({
  ...userActivityLogQuery.list(submitForm),
  placeholderData: keepPreviousData,
});
</script>
<template>
  <search-table
    v-model:page="page"
    v-model:per-page="perPage"
    :is-fetching="isFetching"
    :per-pages="perPages"
    :pagination="data"
  >
    <template #form>
      <el-form inline :model="form" ref="formEl">
        <el-form-item :label="t('common.ip')" prop="ip">
          <el-input v-model="form.ip" clearable />
        </el-form-item>
        <el-form-item
          :label="t('admin.view.userActivityLog.eventType')"
          prop="eventTypes"
        >
          <el-select
            v-model="form.eventTypes"
            clearable
            multiple
            class="w-fit min-w-40"
          >
            <el-option
              v-for="item in typeOptions"
              :key="item.key"
              :value="item.value"
              :label="t(item.label)"
            />
          </el-select>
        </el-form-item>

        <el-form-item
          :label="t('common.user')"
          prop="userIds"
          class="w-fit min-w-40"
        >
          <user-select v-model="form.userIds" multiple />
        </el-form-item>
        <el-form-item
          :label="t('common.client')"
          prop="clientIds"
          class="w-fit min-w-40"
        >
          <client-select v-model="form.clientIds" multiple />
        </el-form-item>
        <el-form-item
          :label="t('common.occurredStartAt')"
          prop="occurredStartAt"
        >
          <el-date-picker
            type="datetime"
            v-model="form.occurredStartAt"
            :disabled-date="
              (date: Date) =>
                form.occurredEndAt
                  ? date.getTime() > form.occurredEndAt.getTime()
                  : false
            "
          />
        </el-form-item>
        <el-form-item :label="t('common.occurredEndAt')" prop="occurredEndAt">
          <el-date-picker
            type="datetime"
            v-model="form.occurredEndAt"
            :disabled-date="
              (date: Date) =>
                form.occurredStartAt
                  ? date.getTime() < form.occurredStartAt.getTime()
                  : false
            "
          />
        </el-form-item>
        <el-form-item>
          <el-button type="primary" @click="search">
            <template #icon>
              <Icon icon="material-symbols:search" />
            </template>
            {{ t("common.search") }}
          </el-button>
          <el-button @click="resetForm">
            <template #icon>
              <Icon icon="carbon:reset" />
            </template>
            {{ t("common.reset") }}
          </el-button>
        </el-form-item>
      </el-form>
    </template>
    <template #table-columns>
      <el-table-column :label="t('common.occurredAt')">
        <template #default="scope">
          {{ d(scope.row.occurredAt, "long") }}
        </template>
      </el-table-column>
      <el-table-column :label="t('admin.view.userActivityLog.eventType')">
        <template #default="scope">
          <el-tag>{{
            t(userActivityLogPresenter.eventTypeText(scope.row.eventType))
          }}</el-tag>
        </template>
      </el-table-column>
      <el-table-column :label="t('common.ip')" prop="ip" />
      <el-table-column :label="t('common.client')" prop="client.name" />
      <el-table-column :label="t('common.user')" prop="user.name" />
    </template>
  </search-table>
</template>
